<?php

declare(strict_types=1);

namespace Drupal\invoice_ninja;

use Drupal\Core\Session\AccountInterface;
use Drupal\invoice_ninja\Form\Settings;

/**
 * Provides services to synchronize data with Invoice Ninja.
 */
final class UserSynchronizer extends SynchronizerBase {

  private const ALLOW_USER_TO_SYNCHRONIZE = 'access invoice_ninja';
  protected const INVOICE_NINJA_COLLECTION_USER = 'invoice_ninja.sync_user';

  /**
   * Synchronize all users with the permission "allow user to synchronize".
   */
  public function synchronizeUsers(): void {
    if ($this->config->get(Settings::SYNCHRONIZE_USERS) !== 1) {
      return;
    }
    $this->getInvoiceNinjaClient()->addHeader([
      'X-API-PASSWORD' =>
      $this->config->get(Settings::INVOICE_NINJA_ADMIN_PASSWORD),
    ]);
    $users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['status' => 1]);
    /** @var \Drupal\Core\Session\AccountInterface $user */
    foreach ($users as $user) {
      if ($user->hasPermission(self::ALLOW_USER_TO_SYNCHRONIZE) &&
        $user->getAccountName() !== 'admin') {
        $clientData = $this->keyValueFactory->get(self::INVOICE_NINJA_COLLECTION_USER)
          ->get((string) $user->id());
        if (!$clientData) {
          $remoteUser = $this->getInvoiceNinjaClient()->users->create($this->getUserData($user));
          $this->storeId(self::INVOICE_NINJA_COLLECTION_USER,
            (string) $user->id(), $remoteUser['data']['id']);
        }
        else {
          $lastSync = $clientData['last_sync'];
          $remoteId = $clientData['remote_id'];
          // @phpstan-ignore-next-line
          if ($user->get("changed")->value > $lastSync) {
            $this->getInvoiceNinjaClient()->users->update($remoteId, $this->getUserData($user));
            $this->storeId(self::INVOICE_NINJA_COLLECTION_USER,
              (string) $user->id(), $remoteId);
          }
        }
      }
    }
  }

  /**
   * Gets the data of a given user.
   *
   * @param \Drupal\Core\Session\AccountInterface $user
   *   The user.
   *
   * @return array
   *   The user data.
   */
  private function getUserData(AccountInterface $user): array {
    return [
      'first_name' => $user->getAccountName(),
      'last_name' => $user->getAccountName(),
      'email' => $user->getEmail(),
      'company_user' => [
        'permissions' => 'create_all,view_all,edit_all',
        'isOwner' => TRUE,
      ],
    ];
  }

}
